import sys, os
from typing import Any
from .remote_interfaces import RemoterInterface
# By AWARE on RC: python3.6 and it happy this *local* version of dataclasses, but ver>3.6 should go with system-wide version
try:
    from dataclasses import dataclass
except:
    from dataclasses_local import dataclass
from pathlib     import Path
import subprocess, shlex

# Requirements:
# cifs-utils:
# Might be a good choise: sudo apt install cifs-utils

class SambaHandlerException(Exception):
    pass

@dataclass
class SambaHandler(RemoterInterface):
    __version__ = "0.0.1"
    internal_name: str  = "[SambaHandler]"
    debug        : bool = False
    verbose      : bool = False
    #
    host         : str  = "192.168.2.52"
    port         : int  = 139
    username     : str  = "user"
    psw          : str  = "psw"
    remote_name  : str  = "samba"        # samba specific thing
    service_name : str  = "service_name" # samba specific thing
    #
    mount_remote_str  : str  = None
    mount_local_path  : Path = None
    build_type        : str  = None

    def __post_init__(self):
        if self.build_type is None:
            raise SambaHandlerException("You have to provide `build_type`(domain)")
        
        if self.mount_remote_str is None:
            raise SambaHandlerException("You have to provide `mount_remote_path`(//192.168.../Public/Dev/...)")
        
        if self.mount_local_path is None:
            raise SambaHandlerException("You have to provide `mount_local_path`(./my/fancy/folder/to/mount)")
        if isinstance(self.mount_local_path, str):
            self.mount_local_path = Path(self.mount_local_path)
        self.mount_local_path = self.mount_local_path.resolve()
        # make folder if not exists
        if not self.mount_local_path.exists():
            self.mount_local_path.mkdir(parents=True)
        else:
            # check it content
            if len(list(self.mount_local_path.iterdir()))> 0:
                # something inside however, no mount yet
                raise SambaHandlerException("You provided a folder with some content. Aborting.")

        # Passmod
    
    def connect(self):
        if self.is_connected():
            pass
        else:
            cmd = f"sudo mount -t cifs {self.mount_remote_str} {self.mount_local_path} -o username={self.username},password={self.psw},noexec 2>&1"
            if self.debug:
                print(f"{self.internal_name} Connection... {cmd}")
            # output = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, encoding="utf-8")
            # if output.stderr: print(f"Connect output: {output.stderr}")
            os.system(cmd)

    '''
    Currently it checks content of the folder
    '''
    def is_connected(self) -> bool:
        return len(list(self.mount_local_path.iterdir())) > 0

    '''
    -l means unmount when would be free
    '''
    def disconnect(self):
        cmd = f"sudo umount -l {self.mount_local_path}"
        
        if self.debug: print(f"{self.internal_name} Disconnecting... {cmd}")
        output = subprocess.run(shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.PIPE, encoding="utf-8")
        if output.stderr: print(f"Disconnect output: {output.stderr}")


    def download(self, src_fname: Path = None, dst_fname: Path = None):
        cmd = f"cp -r {src_fname} {dst_fname}"
        
        if self.debug: print(f"{self.internal_name} Downloading... [{cmd}]")

        output = subprocess.run(shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.PIPE, encoding="utf-8")
        if output.stderr: print(f"Downloading output: {output.stderr}")

    def upload(self, src_fname: Path = None, dst_fname: Path = None):
        pass
        # cmd = f"cp -r {src_fname} {dst_fname}"
        
        # if self.debug: print(f"[{self.internal_name}] Uploading... [{cmd}]")
        # output = subprocess.run(shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.PIPE, encoding="utf-8")
        # if output.stderr: print(f"Uploading output:\n{output.stderr}")

    def exists(self, path: Path) -> bool:
        return super().exists(path)
    
    def list_mount_point(self):
        cmd = f"ls {self.mount_local_path}"
        output = subprocess.run(shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.PIPE, encoding="utf-8")
        if output.stderr: print(f"List output:\n{output.stderr}")
        print("OUT:", output.stdout)


    def execute(self, cmd: str, get_output:bool = False, _silent: bool = False) -> Any :
        if self.debug:
            print(f"{self.internal_name} Execute... {cmd}")
        output = subprocess.run(shlex.split(cmd), stdout=subprocess.PIPE, stderr=subprocess.PIPE, encoding="utf-8")
        
        if output.stderr: print(f"Execute output:\n{output.stderr}")
        if get_output: return output.stdout.split()

    def _execute(self, cmd:str = "pwd", get_output:bool = False, _silent: bool = False, pty=False) -> Any:
        if pty: cmd = f"bash -l -c '{cmd}'"
        stdin_, stdout_, stderr_ = self.ssh.exec_command(cmd)
        stdout_.channel.recv_exit_status()
        if get_output: output = []

        out_lines = stdout_.readlines()
        err_lines = stderr_.readlines()
        for line in [line.strip() for line in out_lines]:
            if not _silent: print(f"->>: {line}")
            if get_output: output.append(line.strip())
        if err_lines:
            for line in [line.strip() for line in err_lines]:
                if not _silent: print(f"-?>: {line}")
                if get_output: output.append(line.strip())
        #return stdin_, stdout_, stderr_
        if get_output: return output
    
    def get_mount_local_path(self):
        return self.mount_local_path
